/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.jbidescriptor;

import java.io.IOException;

import javax.xml.bind.JAXBException;

import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.jbi.descriptor.AbstractJBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.xml.sax.SAXException;

/**
 * CDK JBI descriptor Builder. Provide marshalling/unmarshalling features.
 * @author Christophe HAMERLING - EBM WebSourcing
 * @author Roland NAUDIN - EBM WebSourcing
 */
public class CDKJBIDescriptorBuilder extends AbstractJBIDescriptorBuilder<Jbi> {

    /**
     * CDK schema
     */
    public static final String CDKJBI_XSD = "CDKjbi.xsd";

    /**
     * Extensions schema
     */
    public static final String CDKEXTENSIONS_XSD = "CDKextensions.xsd";

    private static JBIDescriptorException INSTANCE_EXCEPTION;

    private static CDKJBIDescriptorBuilder INSTANCE;

    public static synchronized CDKJBIDescriptorBuilder getInstance() throws JBIDescriptorException {
        if (INSTANCE_EXCEPTION != null) {
            throw INSTANCE_EXCEPTION;
        }
        if (INSTANCE == null) {
            try {
                INSTANCE = new CDKJBIDescriptorBuilder();
            } catch (JBIDescriptorException e) {
                INSTANCE_EXCEPTION = e;
                throw INSTANCE_EXCEPTION;
            }
        }
        return INSTANCE;
    }

    /**
     * Free singleton resources.
     * 
     * If the {@link CDKJBIDescriptorBuilder} is used after to be freed, resources will be re-allocated.
     */
    public static synchronized void destroyInstance() {
        INSTANCE_EXCEPTION = null;
        INSTANCE = null;
    }

    protected CDKJBIDescriptorBuilder() throws JBIDescriptorException {
        super(Jbi.class);
        try {
            // order of xsd matters because jbi imports extensions
            super.init(Jbi.class, CDKEXTENSIONS_XSD, CDKJBI_XSD);
        } catch (final IOException e) {
            throw new JBIDescriptorException("Failed to get resources from the current class-path", e);
        } catch (final SAXException e) {
            throw new JBIDescriptorException("Failed to parse resources", e);
        } catch (JAXBException e) {
            throw new JBIDescriptorException("Failed to create JAXB objects", e);
        }

    }
}
