/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.jbidescriptor;

import static org.junit.Assert.assertEquals;

import java.io.InputStreamReader;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;

import org.junit.Test;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.w3c.dom.Document;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class JBIDescriptorBuilderTest {

    @Test
    public void testComponentDescValidating() throws Exception {
        URL xmlURL = this.getClass().getResource("/component.xml");
        CDKJBIDescriptorBuilder.getInstance().validateJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testComponentDescJavaBuilding_000() throws Exception {
        URL xmlURL = this.getClass().getResource("/component.xml");
        CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testComponentDescJavaBuilding_001() throws Exception {
        URL xmlURL = this.getClass().getResource("/component.xml");

        CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(new InputStreamReader(xmlURL.openStream()));
    }

    
    @Test
    public void testComponentDescXMLBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/component.xml");

        Jbi jbi = CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        CDKJBIDescriptorBuilder.getInstance().writeXmlJBIdescriptor(jbi, document);

        assertEquals(document.getFirstChild().getFirstChild().getLocalName(), "component");
    }

    @Test(expected = JBIDescriptorException.class)
    public void testComponentMissingElement() throws Exception {
        URL xmlURL = this.getClass().getResource("/component-missing-element.xml");
        CDKJBIDescriptorBuilder.getInstance().validateJBIDescriptor(xmlURL.openStream());
    }

    @Test(expected = JBIDescriptorException.class)
    public void testComponentIncorrectValue() throws Exception {
        URL xmlURL = this.getClass().getResource("/component-incorrect-value.xml");
        CDKJBIDescriptorBuilder.getInstance().validateJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testSUProvideDescValidating() throws Exception {
        URL xmlURL = this.getClass().getResource("/su-provide.xml");
        CDKJBIDescriptorBuilder.getInstance().validateJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testSUProvideDescJavaBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/su-provide.xml");

        CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testSUProvideDescXmlBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/su-provide.xml");

        Jbi jbi = CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        CDKJBIDescriptorBuilder.getInstance().writeXmlJBIdescriptor(jbi, document);

        assertEquals(document.getFirstChild().getFirstChild().getFirstChild().getLocalName(), "provides");
    }

    @Test
    public void testSUConsumeDescValidating() throws Exception {
        URL xmlURL = this.getClass().getResource("/su-consume.xml");

        CDKJBIDescriptorBuilder.getInstance().validateJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testSUConsumeDescJavaBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/su-consume.xml");

        CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testSUConsumeDescXmlBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/su-consume.xml");

        Jbi jbi = CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        CDKJBIDescriptorBuilder.getInstance().writeXmlJBIdescriptor(jbi, document);

        assertEquals(document.getFirstChild().getFirstChild().getFirstChild().getLocalName(), "consumes");
    }

    @Test
    public void testDeployableSUProvideDescValidating() throws Exception {
        URL xmlURL = this.getClass().getResource("/deployable-su.xml");
        CDKJBIDescriptorBuilder.getInstance().validateJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testDeployableSUProvideDescJavaBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/deployable-su.xml");

        CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
    }

    @Test
    public void testDeployableSUProvideDescXmlBuilding() throws Exception {
        URL xmlURL = this.getClass().getResource("/deployable-su.xml");

        Jbi jbi = CDKJBIDescriptorBuilder.getInstance().buildJavaJBIDescriptor(xmlURL.openStream());
        Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        CDKJBIDescriptorBuilder.getInstance().writeXmlJBIdescriptor(jbi, document);

        assertEquals(document.getFirstChild().getFirstChild().getFirstChild().getLocalName(), "provides");
    }
}
